#include <signal.h>
#include <be/app/Application.h>
#include <be/app/Message.h>
#include <be/interface/Alert.h>
#include <be/interface/Menu.h>
#include <be/interface/MenuBar.h>
#include <be/interface/MenuItem.h>
#include <be/storage/Directory.h>
#include <be/storage/Entry.h>
#include <be/storage/File.h>
#include <be/storage/FindDirectory.h>
#include <be/storage/Path.h>
#include <be/support/ClassInfo.h>
#include <be/support/Debug.h>
#include "AppDefs.h"
#include "Application.h"
#include "AppView.h"
#include "AppWindow.h"
#include "CommandConstants.h"
#include "GUIStrings.h"
#include "Preferences.h"

#define WIN_LEFT 200
#define WIN_TOP 200
#define WIN_RIGHT 700
#define WIN_BOTTOM 600
#define WIN_MIN_WIDTH 500
#define WIN_MAX_WIDTH 2000
#define WIN_MIN_HEIGHT 400
#define WIN_MAX_HEIGHT 2000

AppWindow::AppWindow() : BWindow(BRect(WIN_LEFT,WIN_TOP,WIN_RIGHT,WIN_BOTTOM),
	WIN_TITLE,B_TITLED_WINDOW,B_ASYNCHRONOUS_CONTROLS)
{
	PRINT(("AppWindow::AppWindow()\n"));

	InitWindow();
}

AppWindow::AppWindow(BMessage* archive) : BWindow(archive)
{
	PRINT(("AppWindow::AppWindow(BMessage*)\n"));

	InitWindow();
}

AppWindow::~AppWindow()
{
	PRINT(("AppWindow::~AppWindow()\n"));
}

void
AppWindow::InitWindow()
{
	PRINT(("AppWindow::InitWindow()\n"));

	SetTitle(WIN_TITLE);
	SetType(B_TITLED_WINDOW);
	SetFlags(B_ASYNCHRONOUS_CONTROLS);
	SetSizeLimits(WIN_MIN_WIDTH,WIN_MAX_WIDTH,WIN_MIN_HEIGHT,WIN_MAX_HEIGHT);
	SetWorkspaces(B_CURRENT_WORKSPACE);

	BMenuBar* menuBar = new BMenuBar(BRect(),"menuBar");

	//create file menu
	BMenu* fileMenu = new BMenu(FILE_MENU);
	fileMenu->AddItem(new BMenuItem(ABOUT_ITEM,
				new BMessage(B_ABOUT_REQUESTED)));
	fileMenu->AddSeparatorItem();
	fileMenu->AddItem(new BMenuItem(QUIT_ITEM,
				new BMessage(B_QUIT_REQUESTED),QUIT_SHORTCUT));
	menuBar->AddItem(fileMenu);

	//create edit menu
	BMenu* editMenu = new BMenu(EDIT_MENU);
	editMenu->AddItem(new BMenuItem(CLEAR_LIST_ITEM,
				new BMessage(CLEAR_LIST),CLEAR_LIST_SHORTCUT));
	editMenu->AddItem(new BMenuItem(SAVE_WINDOW_SETTINGS_ITEM,
				new BMessage(SAVE_WINDOW_SETTINGS),SAVE_WINDOW_SETTINGS_SHORTCUT));
	editMenu->AddItem(new BMenuItem(PREFERENCES_ITEM,
				new BMessage(PREFERENCES),PREFERENCES_SHORTCUT));
	menuBar->AddItem(editMenu);

	//create view menu
	BMenu* viewMenu = new BMenu(VIEW_MENU);
	viewMenu->AddItem(new BMenuItem(EDITOR_VIEW_NAME,
				new BMessage(CHANGE_VIEW),'1'));
	viewMenu->AddItem(new BMenuItem(TA_VIEW_NAME,
				new BMessage(CHANGE_VIEW),'2'));
	viewMenu->AddItem(new BMenuItem(NA_VIEW_NAME,
				new BMessage(CHANGE_VIEW),'3'));
	menuBar->AddItem(viewMenu);

	AddChild(menuBar);
	SetKeyMenuBar(menuBar);

	BRect frame = Bounds();
	frame.top += (menuBar->Bounds()).Height() + 1;

	appView = new AppView(frame);
	AddChild(appView);

	float wHt = Bounds().Height() - (menuBar->Bounds().Height() + 1);
	float vHt,vWt;
	appView->GetPreferredSize(&vWt,&vHt);

	if(wHt < vHt)
	{
		ResizeBy(0,vHt-wHt);
	}
	SetSizeLimits(WIN_MIN_WIDTH,WIN_MAX_WIDTH,vHt,WIN_MAX_HEIGHT);
}

void
AppWindow::SaveWindowSettings()
{
	PRINT(("AppWindow::SaveWindowSettings()\n"));

	PRINT_OBJECT(Frame());
	BPath path;
	find_directory(B_USER_SETTINGS_DIRECTORY,&path);
	path.Append(COMPANY_DIR);
	path.Append(APPLICATION_DIR);
	create_directory(path.Path(),0755);
	path.Append(WINDOW_FILE);

	BMessage* archive = new BMessage();
	status_t status = Archive(archive,false);
	BFile settings(path.Path(),B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE);
	if((settings.InitCheck() == B_OK) && (status == B_OK))
	{
		archive->Flatten(&settings);
	}
}

void
AppWindow::MessageReceived(BMessage* message)
{
	//PRINT(("AppWindow::MessageReceived(BMessage*)\n"));

	switch(message->what)
	{
		case PREFERENCES:
			prefs->Show();
			break;
		case B_ABOUT_REQUESTED:
			AboutRequested();
			break;
		case CHANGE_VIEW:
			int32 index;
			message->FindInt32("index",&index);
			appView->SelectView(index);
			break;
		case CLEAR_LIST:
			appView->ClearList();
			break;
		case SAVE_WINDOW_SETTINGS:
			SaveWindowSettings();
			break;
		case B_REFS_RECEIVED:
			appView->MessageReceived(message);
			break;
		default:
			BWindow::MessageReceived(message);
	}
}

bool
AppWindow::QuitRequested()
{
	PRINT(("AppWindow::QuitRequested()\n"));
	
	thread_id addrefs = find_thread("AddRefs");
	if(addrefs != B_NAME_NOT_FOUND)
	{
		send_signal(addrefs,SIGQUIT);
	}

	if(is_instance_of(be_app,Application))
	{
		be_app->PostMessage(B_QUIT_REQUESTED);
	}
	else
	{
		if(prefs->Lock())
		{
			prefs->Quit();
		}
		if(Lock())
		{
			Quit();
		}
	}
	return true;
}

void
AppWindow::AboutRequested()
{
	PRINT(("AppWindow::AboutRequested()\n"));
	BString msg;
	msg = APPLICATION;
	msg << " ";
	msg << VERSION;
	msg << "\n";
	msg << COMPANY;
	msg << "\n";
	msg << COMPANY_WWW;
	msg << "\n";
	msg << COMPANY_EMAIL;
	BAlert* alert = new BAlert("AboutBox",msg.String(),ABOUT_BTN);
	alert->Go(NULL);
}



AppWindow*
AppWindow::CreateWindow()
{
	PRINT(("AppWindow::CreateWindow()\n"));

	BPath path;
	BFile settings;
	BMessage archive;
	AppWindow* win = 0;

	find_directory(B_USER_SETTINGS_DIRECTORY,&path);
	path.Append(COMPANY_DIR);
	path.Append(APPLICATION_DIR);
	path.Append(WINDOW_FILE);

	if((settings.SetTo(path.Path(), B_READ_ONLY) == B_OK)
			&& (archive.Unflatten(&settings) == B_OK))
	{
		win = new AppWindow(&archive);
	}
	else
	{
		win = new AppWindow();
	}
	return(win);
}
