#include <be/app/Message.h>
#include <be/interface/Button.h>
#include <be/interface/InterfaceDefs.h>
#include <be/interface/Menu.h>
#include <be/interface/MenuField.h>
#include <be/interface/MenuItem.h>
#include <be/interface/TextControl.h>
#include <be/interface/View.h>
#include <be/storage/Directory.h>
#include <be/storage/File.h>
#include <be/storage/FindDirectory.h>
#include <be/storage/Path.h>
#include <be/support/Debug.h>
#include "AppDefs.h"
#include "CommandConstants.h"
#include "GUIStrings.h"
#include "Preferences.h"

#define PREF_WIDTH 375
#define PREF_HEIGHT 150
#define DEFAULT_PATTERN "/a - /n - /k - /t.mp3"

Preferences* prefs = 0;

Preferences::Preferences() :
	BWindow(BRect(100,100,100+PREF_WIDTH,100+PREF_HEIGHT),PREF_TITLE,
	B_MODAL_WINDOW,B_ASYNCHRONOUS_CONTROLS)
{
	PRINT(("Preferences::Preferences()\n"));

	//set things to initial values
	prefFunction = 0;
	prefNAPattern = DEFAULT_PATTERN;

	InitWindow();
	ResetWidgets();

	prefs = this;
}

Preferences::Preferences(BMessage* archive) : BWindow(archive)
{
	PRINT(("Preferences::Preferences(BMessage*)\n"));

	status_t status;

	//pull stuff out of archive
	status = archive->FindInt32("prefFunction",&prefFunction);
	if(status != B_OK)
	{
		prefFunction = 0;
	}

	status = archive->FindString("prefNAPattern",&prefNAPattern);
	if(status != B_OK)
	{
		prefNAPattern = DEFAULT_PATTERN;
	}

	InitWindow();
	ResetWidgets();

	prefs = this;
}

Preferences::~Preferences()
{
	PRINT(("Preferences::~Preferences()\n"));

	SaveWindow();
}

status_t
Preferences::Archive(BMessage* archive, bool deep=true) const
{
	BWindow::Archive(archive,deep);

	PRINT(("Preferences::Archive(BMessage*,bool)\n"));

	status_t status;

	status = archive->AddInt32("prefFunction",prefFunction);
	if(status != B_OK)
	{
		return status;
	}

	status = archive->AddString("prefNAPattern",prefNAPattern);

	return status;
}

void
Preferences::InitWindow()
{
	PRINT(("Preferences::InitWindow()\n"));

	SetTitle(PREF_TITLE);
	SetType(B_MODAL_WINDOW);
	SetFlags(B_ASYNCHRONOUS_CONTROLS);

	BView* view = new BView(Bounds(),"view",B_FOLLOW_ALL,B_NAVIGABLE_JUMP);
	view->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	AddChild(view);

	BRect frame;
	float divider =
		view->StringWidth(DEFAULT_NA_PATTERN_LABEL) + 5.0;
	int space = 20;

	frame = Bounds();
	frame.InsetBy(10,10);
	frame.bottom = frame.top + 10;
	BMenu* menu = new BMenu("");
	menu->SetLabelFromMarked(true);
	menu->AddItem(new BMenuItem(EDITOR_VIEW_NAME,NULL));
	menu->AddItem(new BMenuItem(TA_VIEW_NAME,NULL));
	menu->AddItem(new BMenuItem(NA_VIEW_NAME,NULL));
	prefFunctionMenuField = new BMenuField(frame,"prefFunctionMenuField",
			DEFAULT_FUNCTION_LABEL,menu);
	prefFunctionMenuField->SetDivider(divider);
	view->AddChild(prefFunctionMenuField);

	frame.OffsetBy(0,frame.Height()+space);
	prefNAPatternTextControl = new BTextControl(frame,
			"prefNAPatternTextControl",DEFAULT_NA_PATTERN_LABEL,"",NULL);
	prefNAPatternTextControl->SetDivider(divider);
	view->AddChild(prefNAPatternTextControl);

	space = 10;
	frame = Bounds();
	frame.InsetBy(10,10);
	frame.left = frame.right - 75;
	frame.top = frame.bottom - 30;
	applyButton = new BButton(frame,"applyButton",APPLY_BTN,
			new BMessage(APPLY));
	view->AddChild(applyButton);
	applyButton->MakeDefault(true);

	frame.OffsetBy(-(frame.Width() + space),0);
	cancelButton = new BButton(frame,"cancelButton",CANCEL_BTN,
			new BMessage(CANCEL));
	view->AddChild(cancelButton);
}

int32
Preferences::GetPrefFunction() const
{
	PRINT(("Preferences::GetPrefFunction()\n"));

	return prefFunction;
}

const char*
Preferences::GetPrefNAPattern() const
{
	PRINT(("Preferences::GetPrefNAPattern()\n"));

	return prefNAPattern.String();
}

void
Preferences::ResetWidgets()
{
	PRINT(("Preferences::ResetWidgets()\n"));

	BMenu* menu = prefFunctionMenuField->Menu();
	(menu->ItemAt(prefFunction))->SetMarked(true);
	prefNAPatternTextControl->SetText(prefNAPattern.String());
}

void
Preferences::ApplyAction()
{
	PRINT(("Preferences::ApplyAction()\n"));

	Hide();
	SaveWindow();
}

void
Preferences::CancelAction()
{
	PRINT(("Preferences::CancelAction()\n"));

	Hide();
	ResetWidgets();
}

void
Preferences::SaveWindow()
{
	PRINT(("Preferences::SaveWindow()\n"));

	BMenu* menu = prefFunctionMenuField->Menu();
	BMenuItem* marked = menu->FindMarked();
	if(marked)
	{
		prefFunction = menu->IndexOf(marked);
	}
	else
	{
		prefFunction = 0;
	}

	if(prefNAPatternTextControl->Text())
	{
		prefNAPattern = prefNAPatternTextControl->Text();
	}
	else
	{
		prefNAPattern = DEFAULT_PATTERN;
	}

	BPath path;
	find_directory(B_USER_SETTINGS_DIRECTORY,&path);
	path.Append(COMPANY_DIR);
	path.Append(APPLICATION_DIR);
	create_directory(path.Path(),0755);
	path.Append(PREFS_FILE);

	BMessage* archive = new BMessage();
	Archive(archive,false);
	BFile settings(path.Path(),B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE);
	if(settings.InitCheck() == B_OK)
	{
		archive->Flatten(&settings);
	}
}

void
Preferences::MessageReceived(BMessage* message)
{
	//PRINT(("Preferences::MessageReceived(BMessage*)\n"));

	switch(message->what)
	{
		case APPLY:
			ApplyAction();
			break;
		case CANCEL:
			CancelAction();
			break;
		default:
			BWindow::MessageReceived(message);
			break;
	}
}

void
Preferences::CreatePrefs()
{
	PRINT(("Preferences::CreatePrefs()\n"));

	BPath path;
	BFile settings;
	BMessage archive;
	status_t status;

	find_directory(B_USER_SETTINGS_DIRECTORY,&path);
	path.Append(COMPANY_DIR);
	path.Append(APPLICATION_DIR);
	path.Append(PREFS_FILE);

	status = settings.SetTo(path.Path(), B_READ_ONLY);
	if((status == B_OK) && (archive.Unflatten(&settings) == B_OK))
	{
		new Preferences(&archive);
	}
	else
	{
		new Preferences();
	}

	prefs->Hide();
	prefs->Show();
}

